#line 1 "include/boost/uuid/uuid_serialize.hpp"
#ifndef BOOST_UUID_UUID_SERIALIZE_HPP_INCLUDED
#define BOOST_UUID_UUID_SERIALIZE_HPP_INCLUDED

// Copyright 2007 Andy Tompkins.
// Distributed under the Boost Software License, Version 1.0.
// https://www.boost.org/LICENSE_1_0.txt

// This header is no longer needed and is only retained
// for backward compatibility.

#include <boost/uuid/uuid.hpp>

#endif // BOOST_UUID_UUID_SERIALIZE_HPP_INCLUDED
