#line 1 "include/boost/qvm/vec_register.hpp"
#ifndef BOOST_QVM_VEC_REGISTER_HPP_INCLUDED
#define BOOST_QVM_VEC_REGISTER_HPP_INCLUDED

// Copyright 2008-2024 Emil Dotchevski and Reverge Studios, Inc.
// Copyright 2018 agate-pris
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <boost/qvm/vec_register2.hpp>
#include <boost/qvm/vec_register3.hpp>
#include <boost/qvm/vec_register4.hpp>

#endif
