#line 1 "include/boost/process/child.hpp"
// Copyright (c) 2024 Klemens D. Morgenstern
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)


#include <boost/config/header_deprecated.hpp>
BOOST_HEADER_DEPRECATED("<boost/process/v1/child.hpp>")
#include <boost/process/v1/child.hpp>
