#line 1 "include/boost/spirit/home/lex/lexer_static_lexertl.hpp"
//  Copyright (c) 2001-2011 Hartmut Kaiser
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if !defined(BOOST_SPIRIT_LEX_STATIC_LEXERTL_FEB_11_2008_1045AM)
#define BOOST_SPIRIT_LEX_STATIC_LEXERTL_FEB_11_2008_1045AM

#if defined(_MSC_VER)
#pragma once
#endif

//  These includes make available everything needed to use lexertl either 
//  standalone or as a lexer component for spirit::qi
#include <boost/spirit/home/lex.hpp>
#include <boost/spirit/home/lex/lexer/lexertl/static_lexer.hpp>

#endif
